package Sections2html;

use strict;
use CheckPoint;
use Constants;
use object2html;
use Options;
use Strings;

require Exporter;
our (@ISA, @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(Topology NAT VPN Firewall1_GX Advanced WebServer MailServer DNSServer);

# Initialize special marker signs ...
my $Check      = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/check.png>';
my $CheckTick  = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/checktick.png>';
my $Mark       = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/mark.png>';
my $MarkDot    = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/markdot.png>';
my $Spacer     = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/spacer.png>';
my $None       = '<I>(none)</I>';

##########################################################################
# Routine: Topology
#
# Description:
# Routine to create the Topology section. This section is used in 
# several object types. 
#
# Parameters:
# $rb_name    Rulebase name
# $obj        Reference to object in CheckPoint database
# $name       Name of the object
# $type       Type of the object as determined in Constants.pm
# $state      State of the section (open or closed)
#
# Returns:
# @section    An array with the section to be imported directly
#             in the templates.pm library
##########################################################################
sub Topology {
	my $rb_name = shift;
	my $obj     = shift;
	my $name    = shift;
	my $type    = shift;
	my $state   = shift;
	
	my (@topology);
	my (@interfaces, @routes, @vpndomain);
	my ($allowed_ip, $gateway, $default_gw);
	my (%virtual_router, %interface);
	my (%list, %var);
	my (@var, @val, $var, $val, $v1);
	my ($vs_slot_obj);
	
	my $class  = GetKey($obj, "AdminInfo:ClassName");

	#------------------------------------------------
	# Interfaces ...
	#------------------------------------------------
	@var = GetMembers(GetKey($obj, 'interfaces'));
	if (@var) {
		
		#------------------------------------------------
		# VSX objects have different Interfaces table ...
		#------------------------------------------------
		if (($class =~ /^vs/) && ($class ne "vsx_box")) {

			# Different tables for routing objects ...			
			if (GetKey($obj, "junction") eq "true") {
				push @interfaces, {	
					HEAD => 1, 
					OBJ_VAL1 => html('Name'), 
					OBJ_VAL2 => html('VLAN'),
					OBJ_VAL3 => html('IP address'), 
					OBJ_VAL4 => html('Allowed IP')
				};
				
			} else {		
				# Virtual Router information added ...
				push @interfaces, {	
					HEAD => 1, 
					OBJ_VAL1 => html('Name'), 
					OBJ_VAL2 => html('VLAN'),
					OBJ_VAL3 => html('Virtual Router'),
					OBJ_VAL4 => html('IP address'), 
					OBJ_VAL5 => html('Allowed IP')
				};
			}

			# Find matching object in vs_slot_objects table ...
			$vs_slot_obj = GetKey("vs_slot_objects:".$name);

			# Create hash of interface name to virtual router object ...
			# (all found in vs_slot_obj ...)
			foreach $var (GetMembers(GetKey($vs_slot_obj, 'interfaces'))) {
	
				# Store reference to interface of this name ...
				$val = GetKey($var, 'name').(GetKey($var, 'unique_id')?'.'.GetKey($var, 'unique_id:id'):'');
				$interface{$val} = $var;

				# Store virtual router objects (if exists) ...
				if (GetKey($var, 'junction')) {	
					# Create virtual router object ....
					%var = %{obj2html($rb_name, "network_objects", GetKey($var, 'junction:Name'), $_HREF)};
					$virtual_router{$val} = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				}
			}
			
			# Find all interfaces and sort them ...		
			foreach $var (@var) {
				$list{GetKey($var, 'officialname')}=$var;
			}					
			foreach $var (sort keys %list) {
				# Collect AntiSpoofing information ...
				$var = $list{$var};
				%var = {};
				if (($type =~ /Cluster Member/) && (GetKey($var, 'shared') eq 'true')) {
					%var = %{txt2html('Cluster', '', 'black')};
				} elsif (GetKey($var, 'netaccess:leads_to_internet') eq 'true') {
					%var = %{txt2html('External', 'any_object', 'black')};
				} elsif (GetKey($var, 'netaccess:access') eq 'this') {
					%var = %{txt2html('This Network', 'network', 'black')};
				} elsif (GetKey($var, 'netaccess:access') eq 'specific') {
					%var = %{obj2html($rb_name, GetKey($var, 'netaccess:allowed:Table'), GetKey($var, 'netaccess:allowed:Name'), $_HREF)};
				} elsif (GetKey($var, 'netaccess:access') eq 'undefined') {
					%var = %{txt2html('Undefined', '', 'black')};
				}
				$allowed_ip = $var{OBJ_TEXT};

				if (GetKey($obj, "junction") eq "true") {
					push @interfaces, {
						OBJ_VAL1 => GetKey($var, 'officialname'), 
						OBJ_VAL2 => GetKey($interface{GetKey($var, 'officialname')}, 'unique_id:id') || "&nbsp;",
						OBJ_VAL3 => GetKey($var, 'ipaddr').'/'.$Subnet{GetKey($var, 'netmask')},
						OBJ_VAL4 => $allowed_ip
					}
				} else {
					# Determine Virtual Router ...
					push @interfaces, {
						OBJ_VAL1 => GetKey($var, 'officialname'), 
						OBJ_VAL2 => GetKey($interface{GetKey($var, 'officialname')}, 'unique_id:id') || "&nbsp;",
						OBJ_VAL3 => $virtual_router{GetKey($var, 'officialname')} || "&nbsp;",
						OBJ_VAL4 => GetKey($var, 'ipaddr').'/'.$Subnet{GetKey($var, 'netmask')},
						OBJ_VAL5 => $allowed_ip
					}
				}
			}


		#------------------------------------------------
		# CheckPoint hosts or nodes have no antispoofing
		# information ...
		#------------------------------------------------
		} elsif ($type =~ /Node|Host/) {
			# Without AntiSpoofing information ...
			push @interfaces, {	
				HEAD => 1, 
				OBJ_VAL1 => html('Name'), 
				OBJ_VAL2 => html('IP address'), 
			};
			foreach $var (@var) {
				$list{GetKey($var, 'officialname')}=$var;
			}					
			foreach $var (sort keys %list) {
				$var = $list{$var};
				push @interfaces, {
					OBJ_VAL1 => html(stripquotes(GetKey($var, 'officialname'))), 
					OBJ_VAL2 => GetKey($var, 'ipaddr').'/'.$Subnet{GetKey($var, 'netmask')}
				}
			}
			
		#------------------------------------------------
		# Other objects do have antispoofing information ...
		#------------------------------------------------
		} else {
			# With AntiSpoofing information ...
			push @interfaces, {	
				HEAD => 1, 
				OBJ_VAL1 => html('Name'), 
				OBJ_VAL2 => html('IP address'), 
				OBJ_VAL3 => html('Allowed IP')
			};
		
			foreach $var (@var) {
				$list{GetKey($var, 'officialname')}=$var;
			}					
			foreach $var (sort keys %list) {
				# Collect AntiSpoofing information ...
				$var = $list{$var};
				%var = {};
				$val = '';
				if (($type =~ /Cluster Member/) && (GetKey($var, 'shared') eq 'true')) {
					%var = %{txt2html('Cluster', '', 'black')};
				} elsif (GetKey($var, 'netaccess:leads_to_internet') eq 'true') {
					%var = %{txt2html('External', 'any_object', 'black')};
				} elsif (GetKey($var, 'netaccess:access') eq 'this') {
					%var = %{txt2html('This Network', 'network', 'black')};
				} elsif (GetKey($var, 'netaccess:access') eq 'specific') {
					%var = %{obj2html($rb_name, GetKey($var, 'netaccess:allowed:Table'), GetKey($var, 'netaccess:allowed:Name'), $_HREF)};
				} elsif (GetKey($var, 'netaccess:access') eq 'undefined') {
					%var = %{txt2html('Undefined', '', 'black')};
				}
				if (!$val) { $val = $var{OBJ_TEXT} }
				
				push @interfaces, {
					OBJ_VAL1 => html(stripquotes(GetKey($var, 'officialname'))), 
					OBJ_VAL2 => GetKey($var, 'ipaddr').'/'.$Subnet{GetKey($var, 'netmask')},
					OBJ_VAL3 => $val
				}
			}
		}
	}


	#------------------------------------------------
	# Routes ...
	# (only on VSX objects)
	#------------------------------------------------
	if (($class =~ /^vs/) && ($class ne "vsx_box")) {
		# Find matching object in vs_slot_objects table ...
		$vs_slot_obj = GetKey("vs_slot_objects:".$name);

		# Find routes in vs_slot_object ...
		@var = GetMembers(GetKey($vs_slot_obj, 'routes'));
		if (@var) {
			push @routes, {	
					HEAD => 1, 
					OBJ_VAL1 => html('Destination'), 
					OBJ_VAL2 => html('Gateway'),
					OBJ_VAL3 => html('Interface'),
					OBJ_VAL4 => html('Metric'), 
					OBJ_VAL5 => html('Propagate')
			};

			foreach $var (@var) {
				my $gateway;
				if (GetKey($var, 'next_hop')) {
					$gateway = GetKey($var, 'next_hop');
				} else {
					$gateway = $virtual_router{GetKey($var, 'interface_name')};
				}

				push @routes, {
					OBJ_VAL1 => GetKey($var, 'destination:ipaddr').'/'.$Subnet{GetKey($var, 'destination:netmask')},
					OBJ_VAL2 => $gateway || "&nbsp;",
					OBJ_VAL3 => html(stripquotes(GetKey($var, 'interface_name'))),
					OBJ_VAL4 => "&nbsp;".GetKey($var, 'metric'),
					OBJ_VAL5 => (GetKey($var, 'export') eq "true")?"Yes":"No"
				}
			}
		}
		
		# Determine default gateway ...
		$default_gw = GetKey($vs_slot_obj, 'default_gw_ip_addr') || $virtual_router{GetKey($vs_slot_obj, 'default_gw_interface')};
	}		
		

	#------------------------------------------------
	# VPN Domain ...
	#------------------------------------------------
	if ((GetKey($obj, 'VPN_1') eq 'true') || (GetKey($obj, 'third_party_encryption') eq 'true')) {
		if ($type =~ /Host/) {
			push @vpndomain, {	OBJ_VAL1 => $CheckTick,
								OBJ_VAL2 => html('This Host') };
								
		} else {
			%var = %{obj2html($rb_name, GetKey($obj, 'manual_encdomain:Table'), GetKey($obj, 'manual_encdomain:Name'), $_HREF)};
			$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
			
			if ($class =~ /^vs.*cluster/) {
				$val = "All IP Addresses behind Cluster Members based on Topology information.";
			} elsif ($class =~ /^vs/) {
				$val = "All IP Addresses behind Gateway based on Topology information.";
			} else {
				$val = "All IP addresses behind interfaces";
			}
			push @vpndomain, {	OBJ_VAL1 => (GetKey($obj, 'encdomain') eq 'addresses_behind_gw')?$MarkDot:$Mark,
								OBJ_VAL2 => html($val) },
							 {	OBJ_VAL1 => (GetKey($obj, 'encdomain') eq 'manual')?$MarkDot:$Mark,
								OBJ_VAL2 => html('Manually defined ').
											((GetKey($obj, 'encdomain') eq 'manual')?$var:$Spacer) };
		}
		
	} elsif ((GetKey($obj, 'type') eq 'sofaware_gateway') &&
			 (GetKey($obj, 'sofaware_vpn') eq 'true') && 
			 (GetKey($obj, 'sofaware_mode') eq 'SiteToSite')) {
		%var = %{obj2html($rb_name, GetKey($obj, 'manual_encdomain:Table'), GetKey($obj, 'manual_encdomain:Name'), $_HREF)};
		$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
		push @vpndomain, {	OBJ_VAL1 => (GetKey($obj, 'encdomain') eq 'addresses_behind_gw')?$MarkDot:$Mark,
							OBJ_VAL2 => html('This gateway') },
						 {	OBJ_VAL1 => (GetKey($obj, 'encdomain') eq 'manual')?$MarkDot:$Mark,
							OBJ_VAL2 => html('Manually defined ').
										((GetKey($obj, 'encdomain') eq 'manual')?$var:$Spacer) };
	}

	#------------------------------------------------
	# Combine the subsections ...
	#------------------------------------------------
	if (@interfaces || @routes || @vpndomain) {
		push @topology, {
			SF_BEG => [{	SECTION  => html('Topology'),
							OBJ_NAME => $name,
							STATE	 => ($state eq $_OPEN) }]
		};
		if (@interfaces) {
			push @topology, { 
				SECTION_BEGIN => html('Interfaces'), 
				TABLE => \@interfaces, C5 => 1,
				SECTION_END => 1 
			};
		};
		if (@routes) {
			push @topology, ( 
				{ SECTION_BEGIN => html('Routes') }, 
				{ TABLE => \@routes, C5 => 1 },
				{ SECTION => [
					{	CP2 => 1,
						OBJ_VAL1 => html('Default Gateway:'),
						OBJ_VAL2 => $default_gw	}
				] },
				{ SECTION_END => 1 }
			);
		};
		if (@vpndomain) {
			push @topology, { 
				SECTION_BEGIN => html('VPN Domain'), 
				SECTION => \@vpndomain,
				SECTION_END => 1 
			};
		}
		push @topology, { SF_END => 1 };
	}

	return @topology;
}

##########################################################################
# Routine: NAT
#
# Description:
# Routine to create the NAT section. This section is used in 
# several object types. 
#
# Parameters:
# $rb_name    Rulebase name
# $obj        Reference to object in CheckPoint database
# $name       Name of the object
# $type       Type of the object as determined in Constants.pm
# $state      State of the section (open or closed)
#
# Returns:
# @section    An array with the section to be imported directly
#             in the templates.pm library
##########################################################################
sub NAT {
	my $rb_name = shift;
	my $obj     = shift;
	my $name    = shift;
	my $type    = shift;
	my $state   = shift;
	
	my $section;
	my (@section, @section1, @section2);
	my (%list, %var);
	my (@var, $var, $val, @communities);
	my ($gateway, $CP_vpn, $sofaware_vpn, $external_vpn);

	my $class  = GetKey($obj, "AdminInfo:ClassName");

	#------------------------------------------------
	# NAT configuration
	#------------------------------------------------
	if ((GetKey($obj, 'add_adtr_rule')) &&
		!(
			($class =~ /sofaware/) || ($class =~ /interspect/) || ($class =~ /connectra/) ||
			(GetKey($obj, 'junction') eq 'true')) 
		){

    	push @section, (
    		{ SF_BEG => [
    			{	SECTION  => html('NAT'),
    				OBJ_NAME => $name,
    				STATE	 => ($state eq $_OPEN) }
    		] }
    	);
    	push @section, (
    		{ SECTION_BEGIN => html("Values for Address Translation") },
    		{ SECTION => [ 
    			{	OBJ_VAL1 => (GetKey($obj, 'add_adtr_rule') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html("Add Automatic Address Translation rules") },
    		] }
    	);
    
    	if (GetKey($obj, 'add_adtr_rule') eq 'true') {
    		if (GetKey($obj, 'netobj_adtr_method') eq 'adtr_static') {
    			push @section, (
    				{ SECTION => [
    					{ 	CP2 => 1, 
    						OBJ_VAL1 => html('Translation method:'), 
    						OBJ_VAL2 => html('Static') },
    					{ 	CP2 => 1, 
    						OBJ_VAL1 => html('Translate to IP address:'), 
    						OBJ_VAL2 => GetKey($obj, 'valid_ipaddr') }
    				] }
    			);
    		} else {
    			push @section, (
    				{ SECTION => [
    					{ 	CP2 => 1, 
    						OBJ_VAL1 => html('Translation method:'), 
    						OBJ_VAL2 => html('Hide') }
    				] },
    				{ SECTION => [
    					{ 	OBJ_VAL1 => $Spacer,	
    						OBJ_VAL2 => (GetKey($obj, 'valid_ipaddr') eq '0.0.0.0')?$MarkDot:$Mark, 
    						OBJ_VAL3 => html('Hide behind Gateway') },
    					{ 	CP4 => 1,
    					 	OBJ_VAL1 => $Spacer,	
    						OBJ_VAL2 => (GetKey($obj, 'valid_ipaddr') ne '0.0.0.0')?$MarkDot:$Mark, 
    						OBJ_VAL3 => html('Hide behind IP Address'),
    						OBJ_VAL4 => (GetKey($obj, 'valid_ipaddr') ne '0.0.0.0')?html(GetKey($obj, 'valid_ipaddr')):'' }
    				] }
    			);
    		}
    		%var = %{obj2html($rb_name, GetKey($obj, 'the_firewalling_obj:Table'), GetKey($obj, 'the_firewalling_obj:Name'), $_HREF)};
    		$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    		push @section, (
    			{ SECTION => [
    				{ 	CP2 => 1, 
    					OBJ_VAL1 => html('Install on Gateway:'), 
    					OBJ_VAL2 => $var||"" }
    			] },
    			{ SECTION => [
    				{	OBJ_VAL1 => (GetKey($obj, 'apply_nat_for_cp_conns') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html("Apply for VPN-1 Pro/Express control connections") },
    			] },
    		);
    	}
    	push @section, { SECTION_END => 1 };
    	
    	if (GetKey($obj, 'hide_internal_interfaces')) {
    		push @section, (
    			{ SECTION_BEGIN => html("Automatice Hide NAT for Internal Networks") },
    			{ SECTION => [
    				{	OBJ_VAL1 => (GetKey($obj, 'hide_internal_interfaces') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html("Hide all connections from internal interfaces;").
    								html("to external interfaces behind the gateway") },
    			] },
    			{ SECTION_END => 1 }
    		);
    	}
    
    	# Display IP Pool NAT properties only if this is allowed in the
    	# Global Properties  AND this object does support the property ...
    	if ((GetKey('properties', 'firewall_properties:enable_ip_pool') eq 'true') &&
    		((GetKey($obj, 'support_ip_pool_nat') eq 'true') || GetKey($obj, 'ip_pool_nat'))) {
    
    		push @section, (
    			{ SF_BEG => [
    				{	SECTION  => html('IP Pool NAT'),
    					SUB      => 1,
    					OBJ_NAME => $name,
    					STATE	 => 0 }
    				] },
    		);
    		if (Version($obj) ge "6") {
    			push @section, (
    				{ SECTION => [
    					{ 	OBJ_VAL1 => (GetKey($obj, 'support_ip_pool_nat') eq 'false')?$MarkDot:$Mark,
    						OBJ_VAL2 => html("Do not use IP Pool NAT") }
    				] },
    			);
    			
    			if ($class !~ /cluster/) {
    				%var = %{obj2html($rb_name, GetKey($obj, 'ip_pool_securemote_allocation_name:Table'), GetKey($obj, 'ip_pool_securemote_allocation_name:Name'), $_HREF)};
    				$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    				push @section, (
    					{ SECTION => [
    						{ 	OBJ_VAL1 => (GetKey($obj, 'ip_pool_securemote_allocation_name') eq 'true')?$MarkDot:$Mark,
    							OBJ_VAL2 => html("Allocate IP Addresses from:"),
    							OBJ_VAL3 => $var },
    						{ 	OBJ_VAL1 => (GetKey($obj, 'ip_pool_per_interface') eq 'true')?$MarkDot:$Mark,
    							OBJ_VAL2 => html("Define IP Pool NAT on Gateway interfaces") }
    					] }
    				);
    			} elsif (($class =~ /cluster/) && ($class !~ /member/)) {
    				push @section, (
    					{ SECTION => [
    						{ 	OBJ_VAL1 => (GetKey($obj, 'support_ip_pool_nat') eq 'true')?$MarkDot:$Mark,
    							OBJ_VAL2 => html("Define IP Pool NAT on each cluster member"),
    							OBJ_VAL3 => $var },
    						{ 	OBJ_VAL1 => (GetKey($obj, 'ip_pool_per_interface') eq 'true')?$MarkDot:$Mark,
    							OBJ_VAL2 => html("Define IP Pool NAT on cluster member interfaces") }
    					] }
    				);
    			}
    			
    		} elsif ($class =~ /member/) {
    			%var = %{obj2html($rb_name, GetKey($obj, 'ip_pool_securemote_allocation_name:Table'), GetKey($obj, 'ip_pool_securemote_allocation_name:Name'), $_HREF)};
    			$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    			push @section, (
    				{ SECTION => [
    					{ 	OBJ_VAL1 => (GetKey($obj, 'ip_pool_nat') eq 'true')?$MarkDot:$Mark,
    						OBJ_VAL2 => html("Use IP Pool NAT") },
    					{ 	INACTIVE => (GetKey($obj, 'ip_pool_nat') eq 'false'),
    						OBJ_VAL1 => $Spacer,
    						OBJ_VAL2 => html("Allocate IP Addresses from:"),
    						OBJ_VAL3 => $var },
    				] }
    			);
    	
    		} else { 
    			# Object version < 6 ...
    			if (($class =~ /cluster/) && ($class !~ /member/)) {
    				push @section, (
    					{ TXT => [
    						{ 	TEXT  => html("Define IP Pool NAT on each cluster member"),
    							CLASS => 'note' },
    					] }
    				);
    			}
    			if ($class =~ /gateway_ckp/) {
	    			if ((GetKey($obj, 'ip_pool_securemote') eq 'true') || (GetKey($obj, 'ip_pool_gw2gw') eq 'true')) {
        				%var = %{obj2html($rb_name, GetKey($obj, 'ip_pool_securemote_allocation_name:Table'), GetKey($obj, 'ip_pool_securemote_allocation_name:Name'), $_HREF)};
        				$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
        				push @section, (
        					{ SECTION => [
        						{ 	OBJ_VAL1 => html("Allocate IP Addresses from:"),
        							OBJ_VAL2 => $var },
        					] }
        				);
    				}
    			}
    		}
    		
    				
    		if ((Version($obj) ge "6") || (GetKey($obj, 'VPN_1') eq 'true')) {
    			push @section, (
    				{ SECTION => [
    					{ 	OBJ_VAL1 => (GetKey($obj, 'ip_pool_securemote') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Use IP Pool NAT for VPN clients connections") },
    					{ 	OBJ_VAL1 => (GetKey($obj, 'ip_pool_gw2gw') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Use IP Pool NAT for gateway to gateway connections") }
    				] }
    			);
    		}
    		if (Version($obj) ge "6") {
    			push @section, (
    				{ SECTION => [
    					{ 	OBJ_VAL1 => (GetKey($obj, 'ip_pool_override_hide') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Prefer IP Pool NAT over Hide NAT") },
    				] }
    			);
    		}
	    	
	    	# IP Pool NAT Advanced ...
    		if ( (Version($obj) ge "6") || 
    			 ( (GetKey($obj, 'VPN_1') eq 'true') &&
    			   ((GetKey($obj, 'ip_pool_securemote') eq 'true') || (GetKey($obj, 'ip_pool_gw2gw') eq 'true')) ) ) {
	    		push @section, (
    				{ SF_BEG => [
    					{	SECTION  => html('Advanced...'),
    						BUTTON   => 1,
    						OBJ_NAME => $name."NAT", # Make sure name of Advanced... section is unique within object ...
    						STATE	 => 0 }
    					] },
    				{ SECTION => [
    					{ 	CP2 => 1,
    						OBJ_VAL1 => html("Return unused addresses to IP Pool after:"),
    						OBJ_VAL2 => GetKey($obj, 'ip_pool_unused_return_interval'),
    						OBJ_VAL3 => html("Min") },
    				] },
    				{ SECTION => [
    					{	INACTIVE => (Version($obj) lt "6"),
    						OBJ_VAL1 => (GetKey($obj, 'ip_pool_alloc_per_destination') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Reuse IP addresses from the pool for different destinations") },
    				] },
    				{ SF_END => 1 }
    			);
  			}
    			
    		push @section, { SF_END => 1 };
    	} # End IP Pool NAT ...
    	
    	push @section, { SF_END => 1 };
    } # End NAT section ...

   	return @section;
}


##########################################################################
# Routine: VPN
#
# Description:
# Routine to create the VPN and VPN Advanced section. This section is used in 
# several object types. 
#
# Parameters:
# $rb_name    Rulebase name
# $obj        Reference to object in CheckPoint database
# $name       Name of the object
# $type       Type of the object as determined in Constants.pm
# $state      State of the section (open or closed)
#
# Returns:
# @section    An array with the section to be imported directly
#             in the templates.pm library
##########################################################################
sub VPN {
	my $rb_name = shift;
	my $obj     = shift;
	my $name    = shift;
	my $type    = shift;
	my $state   = shift;
	
	my $section;
	my (@section, @section1, @section2);
	my (%list, %var);
	my (@var, $var, $val, @communities);

	# Determine version of database ...
	my $version = GetKey('FW1Objects.version');
	if ($version le '5.0') { 
		$version = 'NG';
	} elsif ($version gt '5.0') {
		$version = 'NGX';
	}

	# Define characteristics ...
	my $gateway        = (GetKey($obj, 'type') =~ /gateway/);
	my $cluster_member = (GetKey($obj, 'AdminInfo:ClassName') =~ /member/);
	my $CP_vpn         = ((GetKey($obj, 'VPN_1') eq 'true') || (GetKey($obj, 'vpnddcate') eq 'true'));
	my $sofaware_vpn   = (GetKey($obj, 'sofaware_vpn') eq 'true');
	my $external_vpn   = ((GetKey($obj, 'third_party_encryption') eq 'true') ||
						  ($CP_vpn && (GetKey($obj, 'location') eq 'external')) );


	#--------------------------------------------------------------
	# Special case: cluster members do have a different VPN tab...
	#--------------------------------------------------------------
	if ($cluster_member) {
		%var = %{obj2html($rb_name, GetKey($obj, 'om_ippool:Table'), GetKey($obj, 'om_ippool:Name'), $_HREF)};
		$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
		push @section, (
			{ SF_BEG => [
				{	SECTION  => html('VPN'),
					OBJ_NAME => $name,
					STATE	 => ($state eq $_OPEN) }
			] },
			{ SECTION_BEGIN => html("Office Mode for Remote Access") },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($obj, 'ip_pool_office_mode') eq "true")?$CheckTick:$Check,
					OBJ_VAL2 => html("Offer Manual Office Mode (using IP pool)") }
			] },
			{ SECTION => [
				{	INACTIVE => (GetKey($obj, 'ip_pool_office_mode') eq "false"), C1 =>2,
					OBJ_VAL1 => html("Allocate IP Addresses from network:"),
					OBJ_VAL2 => $var }
			] },
			{ SECTION_END => 1 },
			{ SF_END => 1 }
		);
		return @section;
	}

	#------------------------------------------------
	# VPN tab ...
	#------------------------------------------------
	# Check if VPN is enabled ...
	if ((GetKey($obj, 'VPN_1') ne 'true') && 
		(GetKey($obj, 'sofaware_vpn') ne 'true') && 
		(GetKey($obj, 'third_party_encryption') ne 'true') && 
		(GetKey($obj, 'management') ne 'true') ) {
		return @section;
	}
	
	push @section, {
		SF_BEG => [
			{	SECTION  => html('VPN'),
				OBJ_NAME => $name,
				STATE	 => ($state eq $_OPEN) }
		]
	};

	#------------------------------------------------
	# Determine communities for this gateway ...
	#------------------------------------------------
	foreach $var (keys %{GetKey('communities')}) {
		# Check community for membership
		foreach $val (GetMembers(GetKey('communities:'.$var.':participant_gateways'))) {
			if (GetKey($val, 'Name') eq $name) {
				push @communities, $var;
			}
		}
		foreach $val (GetMembers(GetKey('communities:'.$var.':satellite_gateways'))) {
			if (GetKey($val, 'Name') eq $name) {
				push @communities, $var;
			}
		}
	}

	my @section1;
	if (@communities) {
		# Create HTML list of objects ...
		foreach $var (@communities) {
			%var = %{obj2html($rb_name, 'communities', $var, $_HREF)};
			$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
			push @section1, { OBJ_VAL1 => $var };
		}
	} else {
		push @section1, { OBJ_VAL1 => $None }
	}
			
	push @section, (
		{ SECTION_BEGIN => html('Member of the following VPN Communities:') },
		{ SECTION => \@section1 },
		{ SECTION_END => 1 }
	);
	
	#------------------------------------------------
	# Matching criteria ... (certificates)
	#------------------------------------------------
	if ($external_vpn) {
		push @section, (
			{ SF_BEG => [
				{	SECTION  => html('Certificate Matching Criteria'),
					BUTTON     => 1,
					OBJ_NAME => $name,
					STATE	 => 0 }
			] }
		);
		
		if (GetKey($obj, 'isakmp.allowed_ca')) {
			%var = %{obj2html($rb_name, GetKey($obj, 'isakmp.allowed_ca:Table'), GetKey($obj, 'isakmp.allowed_ca:Name'), $_HREF)};
		} else {
			%var = %{txt2html('Any', 'any_object', 'black')};
		}
		$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
		
		push @section, (
			{ TXT => [
				{	CLASS => 'note_xspc',
					TEXT  => html('Gateway must present a certificate issued by CA:') }
			] },
			{ SECTION => [
				{	OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => $var }
			] }
		);
		# Matching criteria for certificate ...
		foreach $var (GetMembers(GetKey($obj, 'isakmp.matchpeer'))) {
			$var{$var} = 1;
		}
		
		push @section, (
			{	TXT	=>	[{	CLASS => 'note_xspc',
							TEXT  => html('The certificate should match any of the following:') }],
				SECTION	=>	[{	CP3 => 1,
							OBJ_VAL1 => $var{'DN'}?$CheckTick:$Check,
							OBJ_VAL2 => html('DN'),
							OBJ_VAL3 => html(stripquotes(GetKey($obj, 'isakmp.dn'))) },
						 {	C2 => 2,
							OBJ_VAL1 => $var{'"DNS/IP"'}?$CheckTick:$Check,
							OBJ_VAL2 => html('IP Address') },
						 {	CP3 => 1,
							OBJ_VAL1 => $var{'Email'}?$CheckTick:$Check,
							OBJ_VAL2 => html('e-mail'),
							OBJ_VAL3 => html(stripquotes(GetKey($obj, 'isakmp.email'))) }]
			}
		);

		push @section, { SF_END => 1 };							
	}

	#------------------------------------------------
	# Traditional mode configuration ...
	#------------------------------------------------
	if ($CP_vpn || $external_vpn) {
		push @section, (
			{ SF_BEG => [
				{	SECTION  => html('Traditional mode configuration'),
					BUTTON   => 1,
					OBJ_NAME => $name,
					STATE	 => 0 }
			] }
		);
		
		# Key exchange encryption methods supported ...
		foreach $var (GetMembers(GetKey($obj, 'isakmp.encmethods'))) {
			$var{$var} = 1;
		}
		push @section, (
			{ COL_BEG => 1 },
			{ SECTION_BEGIN => html('Key exchange:') },
			{ SECTION => [
				{	INACTIVE => (Version($obj) lt '5'),
					OBJ_VAL1 => ($var{'AES-256'})?$CheckTick:$Check,
					OBJ_VAL2 => html('AES-256') },
				{	OBJ_VAL1 => ($var{'3DES'})?$CheckTick:$Check,
					OBJ_VAL2 => html('3DES') },
				{	OBJ_VAL1 => ($var{'DES'})?$CheckTick:$Check,
					OBJ_VAL2 => html('DES') },
				{	OBJ_VAL1 => ($var{'CAST'})?$CheckTick:$Check,
					OBJ_VAL2 => html('CAST') }
			] },
			{ SECTION_END => 1 }
		);
	
		# Data integrity methods supported ...
		foreach $var (GetMembers(GetKey($obj, 'isakmp.hashmethods'))) {
			$var{$var} = 1;
		}
		push @section, (
			{ COL_NXT => 1 },
			{ SECTION_BEGIN => html('Data integrity:') },
			{ SECTION => [
				{	OBJ_VAL1 => ($var{'MD5'})?$CheckTick:$Check,
					OBJ_VAL2 => html('MD5') },
				{	OBJ_VAL1 => ($var{'SHA1'})?$CheckTick:$Check,
					OBJ_VAL2 => html('SHA1') }
			] },
			{ SECTION_END => 1 },
			{ COL_END => 1 }
		);
		
		# Authentication methods ...
		foreach $var (GetMembers(GetKey($obj, 'isakmp.authmethods'))) {
			$var{$var} = 1;
		}
		push @section, (
			{ SECTION_BEGIN => html('Support authentication methods:') },
			{ SECTION => [
				{	OBJ_VAL1 => ($var{'pre-shared'})?$CheckTick:$Check,
					OBJ_VAL2 => html('Pre-Shared Secret') },
				{	OBJ_VAL1 => ($var{'signatures'})?$CheckTick:$Check,
					OBJ_VAL2 => html('Public Key Signatures') }
			] }
		);

		if (($var{'signatures'}) && !$external_vpn) {
			foreach $var (GetMembers(GetKey($obj, 'isakmp.allowed_cert'))) {
				$var{$var} = 1;
			}
    		push @section, (
    			{ SECTION_BEGIN => html('Allowed certificates') },
    			{ TXT => [
    				{	CLASS => 'note_xspc',
    					TEXT  => html('When negotiating with a locally managed peer gateway:') }
    			] },
    			{ SECTION => [
    			  	{	OBJ_VAL1 => $var{'Any'}?$MarkDot:$Mark,
    					OBJ_VAL2 => html('The gateway can use any of its certificates') },
    				{	OBJ_VAL1 => $var{'defaultCert'}?$MarkDot:$Mark,
    					OBJ_VAL2 => html('The gateway must use a certificate issued by this;Certificate Authority') }
    			] }
    		);
    		if ($var{'defaultCert'}) {
    			%var = %{obj2html($rb_name, GetKey($obj, 'isakmp.allowed_ca:Table'), GetKey($obj, 'isakmp.allowed_ca:Name'), $_HREF)};
    			$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    			push @section, 	{	
    				SECTION => [
    					{	OBJ_VAL1 => $Spacer,
    						OBJ_VAL2 => $var }
    				] 
    			};
    		}
    		push @section, { SECTION_END => 1 };
		}
	
		push @section, (
			{ SECTION_END => 1 },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($obj, 'exportable') eq 'true')?$CheckTick:$Check,
					OBJ_VAL2 => html('Exportable for SecuRemote/SecureClient') }
			] }
		);
		
		#------------------------------------------------
		# Traditional Mode Advanced ...
		#------------------------------------------------
		push @section, (
			{ SF_BEG => [
				{ 	SECTION  => html('Traditional mode Advanced'),
					BUTTON   => 1,
					OBJ_NAME => $name,
					STATE	 => 0 }
			] }
		);
		
		if (Version($obj) ge '5.0') {
    		# Diffie-Hellman groups:
    		foreach $var (GetMembers(GetKey($obj, 'isakmp.phase1_DH_groups'))) {
    			$var{GetKey($var, 'Name')} = 1;
    		}
    		push @section, (
    			{ SECTION_BEGIN => html('Support Diffie-Hellman groups for IKE (phase 1) Security associations') },
    			{ SECTION => [
    				{	OBJ_VAL1 => $var{'"Group 1 (768 bit)"'}?$CheckTick:$Check,
    					OBJ_VAL2 => html('Group 1 (768 bit)') },
    				{	OBJ_VAL1 => $var{'"Group 2 (1024 bit)"'}?$CheckTick:$Check,
    					OBJ_VAL2 => html('Group 2 (1024 bit)') },
    				{	OBJ_VAL1 => $var{'"Group 5 (1536 bit)"'}?$CheckTick:$Check,
    					OBJ_VAL2 => html('Group 5 (1536 bit)') },
    				{	OBJ_VAL1 => $var{'"Group 14 (2048 bit)"'}?$CheckTick:$Check,
    					OBJ_VAL2 => html('Group 14 (2048 bit)') },
    			] },
   				{ SECTION_END => 1 }
    		);
    	}
    	
		if (Version($obj) ge '5.0') {
    		# Rekeying Parameters
    		push @section, (
    			{ SECTION_BEGIN => html('Rekeying Parameters') },
    			{ SECTION => [
    				{	CP2 => 1, C1 => 2,
    					OBJ_VAL1 => html('Renegotiate IKE (phase 1) Security associations every'),
    					OBJ_VAL2 => GetKey($obj, 'isakmp.phase1_rekeying_time'),
    					OBJ_VAL3 => html('Minutes') },
    				{	CP2 => 1, C1 => 2,
    					OBJ_VAL1 => html('Renegotiate IPSEC (phase 2) Security associations every'),
    					OBJ_VAL2 => GetKey($obj, 'isakmp.phase2_rekeying_time'),
    					OBJ_VAL3 => html('Seconds') },
    				{	INACTIVE => (GetKey($obj, 'isakmp.phase2_use_rekeying_kbytes') eq 'false'),
    				 	CP3 => 1,
    				 	OBJ_VAL1 => $CheckTick,
    					OBJ_VAL2 => html('Renegotiate IPSEC (phase 2) Security associations every'),
    					OBJ_VAL3 => GetKey($obj, 'isakmp.phase2_rekeying_kbytes'),
    					OBJ_VAL4 => html('KBytes') },
    				{	INACTIVE => (GetKey($obj, 'isakmp.phase2_use_rekeying_kbytes') eq 'true'),
    				 	OBJ_VAL1 => $Check,
    					OBJ_VAL2 => html('Renegotiate IPSEC (phase 2) Security associations every') }
    			] },
    			{ SECTION_END => 1 }
    		);
    	}
		
		# Misc.
		push @section, (
			{	SECTION_BEGIN => html('Misc'),
				SECTION => [
					{	OBJ_VAL1 => (GetKey($obj, 'ISAKMP_aggressive_support') eq 'true')?$CheckTick:$Check,
						OBJ_VAL2 => html('Support agressive mode') }
				],
				SECTION_END => 1
			}
		);
		# End of Traditional Mode Advanced ...				
		push @section, { SF_END => 1 };

		# End of VPN ...
		push @section, { SF_END => 1 };
	}

	#------------------------------------------------
	# Backup gateways ...
	#------------------------------------------------
	if ($gateway && $CP_vpn) {
		if (GetKey('properties:firewall_properties:enable_mep_configuration') eq 'true') {
    		$var = '';
    		if (GetKey($obj, 'backup_gw') eq 'true') {
    			%var = %{obj2html($rb_name, GetKey($obj, 'backup_gateway:Table'), GetKey($obj, 'backup_gateway:Name'), $_HREF)};
    			$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    		}
    		push @section, (
    			{ SECTION => [
    				{	OBJ_VAL1 => (GetKey($obj, 'backup_gw') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html('Use Backup Gateways:'),
    					OBJ_VAL3 => $var }
    			] }
    		);
		}
	}


	#------------------------------------------------
	# VPN Advanced ...
	#------------------------------------------------
	if ($sofaware_vpn) {
		# Although Sofaware does have VPN parameters, it has not
		# VPN Advanced section...
	} elsif	($version eq "NG") {
		push @section, (
			{ SF_BEG => [
				{	SECTION  => html('VPN Advanced'),
					SUB      => 1,
					OBJ_NAME => $name,
					STATE	 => 0 }
			] },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($obj, 'ISAKMP_subnet_support') eq 'true')?$CheckTick:$Check,
					OBJ_VAL2 => html('Support key exchange for subnets') }
			] }
		);

		if (($gateway && !($external_vpn)) && (Version($obj) ge '5.03')) {
			push @section, (
				{ SECTION_BEGIN => html('Restart Options') },
				{ SECTION => [
					{	OBJ_VAL1 => (GetKey($obj, 'ike_support_crash_recovery_sr') eq 'true')?$CheckTick:$Check,
						OBJ_VAL2 => html('Perform an organized shutdown of tunnels;upon gateway restart') }
				] },
				{ SECTION_END => 1 },	# End of Restart Options
			);
		}
		push @section, { SF_END => 1 };	# End of VPN Advanced (NG)
		
	} elsif ($version eq "NGX") { #version NGX
		push @section, (
			{ SF_BEG => [
				{	SECTION  => html('VPN Advanced'),
					SUB      => 1,
					OBJ_NAME => $name,
					STATE	 => 0 }
			] },
			{ SECTION_BEGIN => html("VPN Tunnel Sharing") },
			{ TXT => [
				{	TEXT => html("Control the number of VPN tunnels opened between peer Gateways"),
					CLASS => 'note' }
			] },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($obj, 'take_tunnel_granularity_from_comm') eq 'true')?$MarkDot:$Mark,
					OBJ_VAL2 => html('Use the community settings') },
				{	OBJ_VAL1 => (GetKey($obj, 'take_tunnel_granularity_from_comm') eq 'false')?$MarkDot:$Mark,
					OBJ_VAL2 => html('Custom settings') },
			] }
		);
		if (GetKey($obj, 'take_tunnel_granularity_from_comm') eq 'false') {
			push @section, (
				{ SECTION => [
					{	OBJ_VAL1 => $Spacer,
						OBJ_VAL2 => ((GetKey($obj, 'ISAKMP_subnet_support') eq 'false') &&
									 (GetKey($obj, 'ISAKMP_universal_support') eq 'false'))?$MarkDot:$Mark,
						OBJ_VAL3 => html("One VPN tunnel per each pair of hosts") },
					{	OBJ_VAL2 => (GetKey($obj, 'ISAKMP_subnet_support') eq 'true')?$MarkDot:$Mark,
						OBJ_VAL3 => html("One VPN tunnel per subnet pair") },
					{	OBJ_VAL2 => (GetKey($obj, 'ISAKMP_universal_support') eq 'true')?$MarkDot:$Mark,
						OBJ_VAL3 => html("One VPN tunnel per Gateway pair") },
				] }
			);
		}
		push @section, (
			{ SECTION_END => 1 },	# End of VPN Tunnel Sharing
		);

		if (($gateway && !($external_vpn)) && (Version($obj) ge '5.03')) {
			push @section, (
				{ SECTION_BEGIN => html('Restart Options') },
				{ SECTION => [
					{	OBJ_VAL1 => (GetKey($obj, 'ike_support_crash_recovery_sr') eq 'true')?$CheckTick:$Check,
						OBJ_VAL2 => html('Perform an organized shutdown of tunnels;upon gateway restart') }
				] },
				{ SECTION_END => 1 },
			);
		}
		
		# Wire mode only availabe for NGX and only for special OS
		if ((Version($obj) ge '6') &&
		    (GetKey($obj, 'os_info:osName') =~ /SecurePlatform|IPSO|Linux/)) {
			push @section, (
				{ SECTION_BEGIN => html('Wire mode') },
				{ SECTION => [
					{	OBJ_VAL1 => (GetKey($obj, 'support_wire_mode') eq 'true')?$CheckTick:$Check,
						OBJ_VAL2 => html('Support Wire mode (and Wire mode routing - route uninspected;').
									html('encrypted traffic in VPN routing configurations)') }
				] }
			);
			
			if (GetKey($obj, 'support_wire_mode') eq 'true') {
				push @section, (
					{ COL_BEG => 1 },
					{ TXT => [
						{ 	TEXT => $Spacer }
					] },
					{ COL_NXT => 1 },
					{ TXT => [
						{	TEXT => html("Select the interfaces where traffic destined to Wire mode;").
    								html("communities will bypass the Firewall"),
							CLASS => 'note_xspc' },
					] },
				);
				
				my @interfaces;
				push @interfaces, {
					HEAD => 1,
					OBJ_VAL1 => "Name",
					OBJ_VAL2 => "IP Address",
					OBJ_VAL3 => "Netmask" 
				};
				foreach $var (GetMembers(GetKey($obj, 'interfaces'))) {
					if (GetKey($var, 'netaccess:internal_type') eq "private_LAN") {
						push @interfaces, {
							OBJ_VAL1 => html(stripquotes(GetKey($var, 'officialname'))),
							OBJ_VAL2 => GetKey($var, 'ipaddr'),
							OBJ_VAL3 => GetKey($var, 'netmask')
						};
					}
				}
				
				push @section, (
					{ TABLE => \@interfaces },
					{ SECTION => [
						{	OBJ_VAL1 => (GetKey($obj, 'fw_wire_log') eq 'true')?$CheckTick:$Check,
							OBJ_VAL2 => html("Log Wire mode traffic") }
					] },
					{ COL_END => 1 }
				);
				
			}
			push @section, { SECTION_END => 1 };	# End of Wire mode
		}
		
		if (Version($obj) ge '6') {
			# if 'ike_support_nat_t' is not defined, it is supposed to be true!!!
			# this appears to be a bug in the software of CheckPoint ...
			push @section, (
				{ SECTION_BEGIN => html("NAT traversal (Industry standard)") },
				{ SECTION => [
					{	OBJ_VAL1 => (GetKey($obj, 'ike_support_nat_t') ne 'false')?$CheckTick:$Check,
						OBJ_VAL2 => html("Support NAT traversal (applies to Remote Access and Site to;").
									html("Site connections)") }
				] },
				{ SECTION_END => 1 }
			);
		}
		push @section, { SF_END => 1 };	# End of VPN Advanced
	}

	#------------------------------------------------
	# Link Selection ...
	# ... only available in NGX ...
	#------------------------------------------------
	if (($gateway) && ($version eq 'NGX')) {
		push @section, (
			{ SF_BEG => [
				{	SECTION  => html('Link Selection'),
					SUB      => 1,
					OBJ_NAME => $name,
					STATE	 => 0 }
			] },
			{ SECTION_BEGIN => html("IP Selection by Remote Peer") },
			{ TXT => [
				{	TEXT => html("Locally managed VPN peers will determine this gateway's IP address;").
							html("using one of the following methods:"),
					CLASS => 'note' }
			] },
		);

		if (GetKey($obj, 'ipaddr') !~ /0.0.0/) {
			#No dynamic IP address ...
			push @section, (
    			{ SECTION => [
    				{	OBJ_VAL1 => (GetKey($obj, 'ip_resolution_mechanism_GW') =~ /IpVpn|IpVPN/)?$MarkDot:$Mark,
    					OBJ_VAL2 => html('Always use this IP address:') },
    					{	INACTIVE => (GetKey($obj, 'ip_resolution_mechanism_GW') !~ /IpVpn|IpVPN/),
    						OBJ_VAL1 => $Spacer,
    						OBJ_VAL2 => ((GetKey($obj, 'ip_resolution_mechanism_GW') eq 'mainIpVpn')?$MarkDot:$Mark).
    									' '.html('Main address') },
    					{	INACTIVE => (GetKey($obj, 'ip_resolution_mechanism_GW') !~ /IpVpn|IpVPN/),  CP3 => 1,
    						OBJ_VAL1 => $Spacer,
    						OBJ_VAL2 => ((GetKey($obj, 'ip_resolution_mechanism_GW') eq 'singleIpVpn')?$MarkDot:$Mark).
    									' '.html('Selected address from topology table:'),
    						OBJ_VAL3 => (GetKey($obj, 'ip_resolution_mechanism_GW') eq 'singleIpVpn')?GetKey($obj, 'single_VPN_IP'):"" },
    					{	INACTIVE => (GetKey($obj, 'ip_resolution_mechanism_GW') !~ /IpVpn|IpVPN/),  CP3 => 1,
    						OBJ_VAL1 => $Spacer,
    						OBJ_VAL2 => ((GetKey($obj, 'ip_resolution_mechanism_GW') eq 'singleNATIpVPN')?$MarkDot:$Mark).
    									' '.html('Statically NATed IP:'),
    						OBJ_VAL3 => (GetKey($obj, 'ip_resolution_mechanism_GW') eq 'singleNATIpVPN')?GetKey($obj, 'single_VPN_IP'):"" },
    				{	OBJ_VAL1 => (GetKey($obj, 'ip_resolution_mechanism_GW') eq 'topologyCalc')?$MarkDot:$Mark,
    					OBJ_VAL2 => html('Calculate IP based on network topology') },
    			] }
    		);
    	}
    	
		push @section, (
   			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($obj, 'ip_resolution_mechanism_GW') =~ /dns/ )?$MarkDot:$Mark,
					OBJ_VAL2 => html('Use DNS resolving:') },
				{	INACTIVE => (GetKey($obj, 'ip_resolution_mechanism_GW') !~ /dns/ ), CP3 => 1,
					OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => ((GetKey($obj, 'ip_resolution_mechanism_GW') eq 'dnsLookup')?$MarkDot:$Mark).
								' '.html('Full hostname:'),
					OBJ_VAL3 => (GetKey($obj, 'ip_resolution_mechanism_GW') eq 'dnsLookup')?GetKey($obj, 'dns_IP_resolution_GW'):"" },
				{	INACTIVE => (GetKey($obj, 'ip_resolution_mechanism_GW') !~ /dns/ ), C2 => 2,
					OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => ((GetKey($obj, 'ip_resolution_mechanism_GW') eq 'dnsComposedLookup')?$MarkDot:$Mark).
								' '.html("Gateway's name and domain name (specified in Global Properties)") },
			] }
		);
		
		if (GetKey($obj, 'ipaddr') =~ /0.0.0/) {
			# Dynamic IP address ...
			push @section, (
   				{ SECTION => [
						{	OBJ_VAL1 => (!GetKey($obj, 'ip_resolution_mechanism_GW') || 
										(GetKey($obj, 'ip_resolution_mechanism_GW') eq "none"))?$MarkDot:$Mark,
							OBJ_VAL2 => html("None (use last known IP address)") },
				] },
			);
		} else {
			# No Dynamic IP address ...
			push @section, (
   				{ SECTION => [
						{	OBJ_VAL1 => (GetKey($obj, 'ip_resolution_mechanism_GW') =~ /Prob/)?$MarkDot:$Mark,
							OBJ_VAL2 => html('Use a probing method:') },
				] },
			);
			
    		# Add DNS probing configuration		
    		if (GetKey($obj, 'ip_resolution_mechanism_GW') =~ /Prob/) {
    			push @section, (
    				{ COL_BEG => 1 },
    				{ TXT => [{ TEXT => $Spacer }] },
    				{ COL_NXT => 1 },
    				{ SECTION => [
    					{	OBJ_VAL1 => (GetKey($obj, 'ip_resolution_mechanism_GW') eq 'ongoingProb')?$MarkDot:$Mark,
    						OBJ_VAL2 => html('Using ongoing probing:') },
    					{	OBJ_VAL1 => (GetKey($obj, 'ip_resolution_mechanism_GW') eq 'oneTimeProb')?$MarkDot:$Mark,
    						OBJ_VAL2 => html('Using one time probing:') },
    				] },
    				{ COL_NXT => 1 },
    			);
    
        		my @ipaddr;
        		push @ipaddr, {
        			HEAD => 1,
        			OBJ_VAL1 => html("IP Address")
        		};
        		if (GetKey($obj, 'available_VPN_IP_list_GW')) {
        			foreach $var (GetMembers(GetKey($obj, 'available_VPN_IP_list_GW'))) {
        				push @ipaddr, { OBJ_VAL1 => $var };
        			}
        		}
    
        		push @section, (
    				{ SF_BEG => [
    					{	SECTION  => html('Configure...'),
    						BUTTON   => 1,
    						OBJ_NAME => $name."Link",
        					STATE	 => 0 }
        			] },
        			{ SECTION => [
        				{	OBJ_VAL1 => (GetKey($obj, 'use_interface_IP_GW') eq 'true')?$MarkDot:$Mark,
        					OBJ_VAL2 => html("Probe all addresses defined in the topology tab") },
        				{	OBJ_VAL1 => (GetKey($obj, 'use_interface_IP_GW') eq 'false')?$MarkDot:$Mark,
        					OBJ_VAL2 => html("Probe the following addresses:") },
        			] },
    
        			{ COL_BEG => 1 },
        			{ TXT => [{ TEXT => $Spacer }] },
        			{ COL_NXT => 1 },
        			{ TABLE => \@ipaddr },
        			{ COL_END => 1 },
        			
        			{ SECTION => [
        				{	CP3 => 1,
        					OBJ_VAL1 => (GetKey($obj, 'interface_resolving_ha_primary_if_GW') ne "")?$CheckTick:$Check,
        					OBJ_VAL2 => html("Primary address"),
        					OBJ_VAL3 => GetKey($obj, 'interface_resolving_ha_primary_if_GW') },
        			] },
        			{ SF_END => 1 }, # End of Configure... button
    
    	    		{ COL_END => 1 }, # End of DNS Probing
        		);
        	}
		} # End of no dynamic IP address
		
    	push @section , { SECTION_END => 1 }; # End of IP Selection by Remote Peer
    	
    	if (Version($obj) ge '6') {
			push @section, (
				{ SECTION_BEGIN => html("Outgoing Route Selection") },
			);
			if (GetKey($obj, 'os_info:osName') =~ /SecurePlatform|IPSO|Linux/) {
				push @section, (
					{ TXT => [
						{	TEXT => html("Determine the outgoing interface using one of the following methods:"),
							CLASS => 'note' },
						{	TEXT => $Spacer.html("When initiating a tunnel"),
							CLASS => 'note_xspc' },
					] },
					{ SECTION => [
						{	OBJ_VAL1 => $Spacer.$Spacer,
							OBJ_VAL2 => (GetKey($obj, 'multiple_ISP_VPN') eq 'false')?$MarkDot:$Mark,
							OBJ_VAL3 => html("Operating system routing table") },
						{	OBJ_VAL2 => (GetKey($obj, 'multiple_ISP_VPN') eq 'true')?$MarkDot:$Mark,
							OBJ_VAL3 => html("Route based probing") },
					] },
				);
				if (GetKey($obj, 'multiple_ISP_VPN') eq 'false') {
					push @section, (
    					{ TXT => [
    						{	TEXT => $Spacer.html("When responding to a remotely initiated tunnel"),
    							CLASS => 'note' },
    					] },
    					{ SECTION => [
    						{	OBJ_VAL1 => $Spacer.$Spacer,
    							OBJ_VAL2 => (GetKey($obj, 'respond_from_same_ifc') eq 'false')?$MarkDot:$Mark,
    							OBJ_VAL3 => html("Use outgoing traffic configuration") },
    						{	OBJ_VAL2 => (GetKey($obj, 'respond_from_same_ifc') eq 'true')?$MarkDot:$Mark,
    							OBJ_VAL3 => html("Reply from the same interface") },
    					] }
    				);
    			}
			} else {
				push @section, (
					{ TXT => [
						{	TEXT => html("When initiating a tunnel the outgoing interface will be selected by the;").
									html("operating system."),
							CLASS => 'note' },
					] },
				);
			}
			
			# This is a new setting not available in all objects yet...
			if (!GetKey($obj, 'outgoing_source_ip')) {
				SetKey($obj, 'outgoing_source_ip', 'automatic');
			}
			push @section, (
				{ SF_BEG => [
					{	SECTION  => html('Source IP address settings...'),
						BUTTON   => 1,
						OBJ_NAME => $name,
    					STATE	 => 0 }
    			] },
    			{ TXT => [
    				{ 	TEXT => html("When initiating a tunnel use the following IP address as the source IP;").
								html("of outgoing packets:"),
						CLASS => 'note' },
				] },
				{ SECTION => [
					{	OBJ_VAL1 => (GetKey($obj, 'outgoing_source_ip') eq 'automatic')?$MarkDot:$Mark,
						OBJ_VAL2 => html("Automatic (derived from method of IP selection by remote peer)") },
					{	OBJ_VAL1 => (GetKey($obj, 'outgoing_source_ip') ne 'automatic')?$MarkDot:$Mark,
						OBJ_VAL2 => html("Manual:") },
				] },
			);
			if (GetKey($obj, 'outgoing_source_ip') ne 'automatic') {
				push @section, (
					{ SECTION =>[
						{	OBJ_VAL1 => $Spacer,
							OBJ_VAL2 => (GetKey($obj, 'outgoing_source_ip') eq 'mainIP')?$MarkDot:$Mark,
							OBJ_VAL3 => html("Main IP address") },
						{	CP4 => 1,
							OBJ_VAL2 => (GetKey($obj, 'outgoing_source_ip') eq 'singleIP')?$MarkDot:$Mark,
							OBJ_VAL3 => html("Selected address from topology table:"),
							OBJ_VAL4 => (GetKey($obj, 'outgoing_source_ip') eq 'singleIP')?GetKey($obj, 'outgoing_single_IP'):"" },
						{	OBJ_VAL2 => (GetKey($obj, 'outgoing_source_ip') eq 'chosenIfcIP')?$MarkDot:$Mark,
							OBJ_VAL3 => html("IP address of chosen interface") },
					] }
				);
			}
			push @section, { SF_END => 1 }; # End of Source IP address settings
						
			push @section, { SECTION_END => 1 }; # End of Outgoing Route Selection				
		}
		# Tracking ...
		push @section, (
			{ SECTION_BEGIN => html("Tracking") },
			{ SECTION => [
				{	CP2 => 1,
					OBJ_VAL1 => html("Outgoing link tracking:"),
					OBJ_VAL2 => $Track{(GetKey($obj, 'vpn_link_resolver_notification')||"none")} },
			] },
			{ SECTION_END => 1 }
		);
		    			
		push @section, { SF_END => 1 };	# End of Link Selection
	}

	push @section , { SF_END => 1 }; # End of VPN
	
	return @section;
	
} # End of sub VPN


##########################################################################
# Routine: Firewall1_GX
#
# Description:
# Routine to create the Firewall-1 GX section. This section is used in 
# several object types. 
#
# Parameters:
# $rb_name    Rulebase name
# $obj        Reference to object in CheckPoint database
# $name       Name of the object
# $type       Type of the object as determined in Constants.pm
# $state      State of the section (open or closed)
#
# Returns:
# @section    An array with the section to be imported directly
#             in the templates.pm library
##########################################################################
sub Firewall1_GX {
	my $rb_name = shift;
	my $obj     = shift;
	my $name    = shift;
	my $type    = shift;
	my $state   = shift;
	
	my (@section);
	
	if (GetKey($obj, 'enforce_gtp_rate_limit') eq 'true') {
		push @section, (
			{ 	SF_BEG => [{	SECTION  => html('FireWall-1 GX'), 
								OBJ_NAME => $name, 
								STATE    => ($state eq $_OPEN) }],
				SECTION =>	[{	CP3 => 1,
							OBJ_VAL1 => $CheckTick,
							OBJ_VAL2 => html('Enforce GTP Signal packet rate limit'),
							OBJ_VAL3 =>	GetKey($obj, 'gtp_rate_limit'),
							OBJ_VAL4 =>	html('PDU/sec') }],
				SF_END => 1
			}
		);
	}
	return @section;
	
} # End of sub Firewall-1_GX


##########################################################################
# Routine: Advanced
#
# Description:
# Routine to create the Advanced section. This section is used in 
# several object types. In most cases only SNMP is involved, sometimes
# more.
#
# Parameters:
# $rb_name    Rulebase name
# $obj        Reference to object in CheckPoint database
# $name       Name of the object
# $type       Type of the object as determined in Constants.pm
# $state      State of the section (open or closed)
#
# Returns:
# @section    An array with the section to be imported directly
#             in the templates.pm library
##########################################################################
sub Advanced {
	my $rb_name = shift;
	my $obj     = shift;
	my $name    = shift;
	my $type    = shift;
	my $state   = shift;
	
	my (@section, @section1);
	my (%var, $var, $obj1);
	
	# Define some general purpose variables ...
	my $FW_Installed = (GetKey($obj, 'firewall') eq 'installed') && 
						(GetKey($obj, 'AdminInfo:ClassName') !~ /sofaware/) && 
						(GetKey($obj, 'location') ne 'external');
	my $VSX_device   = (GetKey($obj, 'AdminInfo:ClassName') =~ /^vs/);
	my $VSX_router   = $VSX_device && (GetKey($obj, 'junction') eq 'true');
	my $OSE_device   = (GetKey($obj, 'AdminInfo:ClassName') =~ /OSE/);

	#-----------------------------------------------------------------
	# Special case: Sofaware devices do have a different Advanced tab
	#-----------------------------------------------------------------
	if (GetKey($obj, 'AdminInfo:ClassName') eq "sofaware_gateway") {
		push @section, (
			{ SF_BEG => [
				{	SECTION  => html('Advanced'), 
					OBJ_NAME => $name, 
					STATE    => ($state eq $_OPEN) }
			] },
			{ SECTION_BEGIN => html('Licensing') },
			{ SECTION => [ 
				{	CP2 => 1, 
					OBJ_VAL1 => html('Product Key:'), 
					OBJ_VAL2 => html(GetKey($obj, 'product_key')) },
				{	CP2 => 1, 
					OBJ_VAL1 => html('MAC Address:'), 
					OBJ_VAL2 => html(GetKey($obj, 'MAC_address')) }
			] },
			{ SECTION_END => 1 },
			{ SF_END => 1 }
		);
	}	

	#-----------------------------------------------------------------
	# Advanced TAB ...
	#-----------------------------------------------------------------
	# Define which sections to display ...
	my $adv_snmp = 	($FW_Installed && !$VSX_device &&
					(GetKey($obj, 'AdminInfo:ClassName') !~ /cluster/)) ||
					(GetKey($obj, 'AdminInfo:ClassName') =~ /host/) || 
					(GetKey($obj, 'AdminInfo:ClassName') =~ /gateway_plain/) || 
					(GetKey($obj, 'AdminInfo:ClassName') =~ /OSE/);
	my $adv_smtp = $FW_Installed && !$VSX_device && 
					(GetKey($obj, 'ipaddr') !~ /0.0.0/) && GetKey($obj, 'smtp');
	my $adv_sam  = $FW_Installed && !$VSX_device && GetKey($obj, 'sam_allow_remote_request');
	my $adv_conn = $FW_Installed && !$VSX_router && GetKey($obj, 'fw_keep_old_conns');
	my $adv_perm = ($FW_Installed || $OSE_device) && !$VSX_router && GetKey($obj, 'AdminInfo:object_permissions:manage');
	my $adv_synd = $FW_Installed && !$VSX_router && (GetKey($obj, 'fwsynatk_method') || GetKey($obj, 'asm_synatk'));

	# SNMP ...
	if ($adv_snmp) {
		push @section1, (
			{ SECTION_BEGIN => html("SNMP") },
			{ SECTION => [
				{	CP2 => 1,
					OBJ_VAL1 => html('sysName:'),
					OBJ_VAL2 => html(stripquotes(GetKey($obj, 'sysName'))) },
				{	CP2 => 1,
					OBJ_VAL1 => html('sysLocation:'),
					OBJ_VAL2 => html(stripquotes(GetKey($obj, 'sysLocation'))) },
				{	CP2 => 1,
					OBJ_VAL1 => html('sysContact:'),
					OBJ_VAL2 => html(stripquotes(GetKey($obj, 'sysContact'))) },
				{	CP2 => 1,
					OBJ_VAL1 => html('sysDescription:'),
					OBJ_VAL2 => html(stripquotes(GetKey($obj, 'sysDescr'))) }
			] },
			{ SECTION_END => 1 }
		);
	}
	
	# SMTP ...
	if ($adv_smtp) {
		if (Version($obj) ge '5.0') {
    		push @section1, (
    			{ SF_BEG => [
    				{	SECTION  => html('SMTP'), 
    					SUB      => 1,
    					OBJ_NAME => $name, 
    					STATE    => 0 }
    			] },
    			{ SECTION => [
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Connection timeout:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:timeout'),
    					OBJ_VAL3 => html('Sec') },
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Dequeuer scan period:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:scan_period'),
    					OBJ_VAL3 => html('Sec') },
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Mail resend period:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:resend_period'),
    					OBJ_VAL3 => html('Sec') },
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Mail abandon time:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:abandon_time'),
    					OBJ_VAL3 => html('Sec') },
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Maximum number of recipients:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:maxrecipients') },
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Maximum generated concurrent connections:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:max_conns') },
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Maximum generated concurrent connections per site:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:max_conns_per_site') },
    				{	CP2 => 1,
    					OBJ_VAL1 => html("Don't accept mail larger than:"),
    					OBJ_VAL2 => GetKey($obj, 'smtp:max_mail_size'),
    					OBJ_VAL3 => html('Kb') },
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Maximum number of mail messages in spool:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:spool_limit') },
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Check if spool goes over/under the limit every:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:spool_limit_scan_period'),
    					OBJ_VAL3 => html('Sec') }
    			] },
    			{ SECTION => [
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Spool Directory:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:rundir')||"-" },
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Postmaster Name:'),
    					OBJ_VAL2 => GetKey($obj, 'smtp:postmaster')||"-" }
    			] },
    			{	SF_END => 1 }
    		);			
		}
	} # End of ... if ($adv_smtp)

	# SAM ...
	if ($adv_sam) {
		push @section1, (
			{ 	SF_BEG => [{	SECTION  => html('SAM'), 
								SUB      => 1,
								OBJ_NAME => $name, 
								STATE    => 0 }] },
			{	TXT	=>	[{	CLASS => 'note',
							TEXT  => html('Using the SAM protocol, configure the manner in which dynamic;').
									 html('restrictions are added to the VPN-1 & FW-1 Rule Base.') }] },
			{ 	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'sam_allow_remote_request') eq 'true')?$CheckTick:$Check,
							OBJ_VAL2 => html("Forward SAM clients' request to other SAM servers") }] }
		);

		my $early = GetKey($obj, 'sam_proxy_backwards_compatibility_security_method');
		push @section1, (
			{	SECTION_BEGIN	=> html('When connecting to this SAM server') },
			{ 	SECTION	=>	[{	OBJ_VAL1 => $early?$CheckTick:$Check,
							OBJ_VAL2 => html("Use early versions compatibility mode") }] }
		);
		if ($early) {
			push @section1, (
				{	SECTION	=>	[{	OBJ_VAL1 => $Spacer,
						 		OBJ_VAL2 => ($early eq 'opsec')?$MarkDot:$Mark,
						 		OBJ_VAL3 => html('Clear (opsec)') },
							 {	OBJ_VAL1 => $Spacer,
						 		OBJ_VAL2 => ($early eq 'auth_opsec')?$MarkDot:$Mark,
						 		OBJ_VAL3 => html('OPSEC Authentication (auth_opsec)') },
							 {	OBJ_VAL1 => $Spacer,
						 		OBJ_VAL2 => ($early eq 'ssl_opsec')?$MarkDot:$Mark,
						 		OBJ_VAL3 => html('OPSEC SSL (ssl_opsec)') },
							 {	OBJ_VAL1 => $Spacer,
						 		OBJ_VAL2 => ($early eq 'ssl_clear_opsec')?$MarkDot:$Mark,
						 		OBJ_VAL3 => html('OPSEC SSL Clear (ssl_clear_opsec)') },
							 ] }
			);
		}
		push @section1, (
			{ 	SECTION	=>	[{	CP3 => 2,
							OBJ_VAL1 => (GetKey($obj, 'sam_enable_purge_history_file') eq 'true')?$CheckTick:$Check,
							OBJ_VAL2 => html('Purge SAM file when it reaches:'),
							OBJ_VAL3 => (GetKey($obj, 'sam_enable_purge_history_file') eq 'true')?GetKey($obj, 'sam_purge_file_start_size'):'-',
							OBJ_VAL4 => html('KBytes') }] }
		);

		push @section1, { SECTION_END => 1 };

		# Close of SAM
		push @section1, { SF_END => 1 };
	} # End of ... if ($adv_sam)
	
	# Connection Persistence ...
	if ($adv_conn) {
		if (Version($obj) ge '5.0') {
			push @section1, (
				{ 	SF_BEG => [{	SECTION  => html('Connection Persistence'), 
									SUB      => 1,
									OBJ_NAME => $name, 
									STATE    => 0 }] },
				{	SECTION	=>	[{	OBJ_VAL1 => html('Handling established connections when installing a new policy:') }] },
				{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'fw_keep_old_conns') eq 'true')?$MarkDot:$Mark,
								OBJ_VAL2 => html('Keep all connections') },
							 {	OBJ_VAL1 => (GetKey($obj, 'save_data_conns') eq 'true')?$MarkDot:$Mark,
								OBJ_VAL2 => html('Keep data connections') },
							 {	OBJ_VAL1 => ((GetKey($obj, 'fw_keep_old_conns') eq 'false') &&
											 (GetKey($obj, 'save_data_conns')   eq 'false'))?$MarkDot:$Mark,
								OBJ_VAL2 => html('Rematch connections') }] },
				{	SF_END => 1 }
			);
		}
	} # End of ... if ($adv_conn)
		
	# Permissions to Install ...
	if ($adv_perm) {
		push @section1, (
			{ 	SF_BEG => [{	SECTION  => html('Permissions to Install'), 
								SUB      => 1,
								OBJ_NAME => $name, 
								STATE    => 0 }] },
			{	TXT	=>	[{	CLASS => 'note',
							TEXT  => html('Only the listed groups are allowed to;install policies on this object.') }] }
		);
		
		$var = HTMLMemberList($rb_name, GetKey($obj, 'AdminInfo:object_permissions:manage'), $_HREF);
		if (!@{$var}) { $var = [{ OBJ_TEXT => $None }]	};
		push @section1, { OBJLIST => $var };
		push @section1, { SF_END => 1 };
	} # End of ... if ($adv_perm)

	# SYNDefender ...
	if ($adv_synd) {
		if (Version($obj) ge '5.0') {
			# SYNDefender >= NG ...
			push @section1, (
				{ SF_BEG => [
					{	SECTION  => html('SYNDefender'), 
						SUB      => 1,
						OBJ_NAME => $name, 
						STATE    => 0 }
				] },
				{ TXT => [
					{	CLASS => 'note_italic',
						TEXT  => html("Warning: All modules' SYNDefender settings might've been;overriden by the SmartDefense configuration") }
				] }
			);

			if (Version($obj) ge '5.03') {
				# SYNDefender >= NG FP3 ...
				push @section1, (
					{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'asm_synatk') eq 'true')?$CheckTick:$Check,
									OBJ_VAL2 => html('Activate SYN Attack protection') }] }
				);
				if (GetKey($obj, 'asm_synatk') eq 'true') {
					push @section1, (
						{	SECTION=>	[{	CP3 => 1, C2 => 2,
										OBJ_VAL1 => $Spacer,
										OBJ_VAL2 => html('Timeout for SYN attack identification:'),
										OBJ_VAL3 => GetKey($obj, 'asm_synatk_timeout'),
										OBJ_VAL4 => html('seconds') },
									 {	OBJ_VAL1 => $Spacer,
										OBJ_VAL2 => (GetKey($obj, 'asm_synatk_external_only') eq 'true')?$CheckTick:$Check,
										OBJ_VAL3 => html('Protect external interface only') }] }
					);
					push @section1, (
						{ SECTION_BEGIN => html('SYN Attack Detection') },
						{ SECTION => [
							{	CP2 => 1,
								OBJ_VAL1 => html('Switch to SYN Relay Defense upon detection of at least;').
											html('SYN packets per timeout'),
								OBJ_VAL2 => GetKey($obj, 'asm_synatk_threshold') }
						] },
						{ SECTION_END => 1 },
						{ SECTION_BEGIN => html('Logging Options') },
						{ SECTION => [
							{	CP2 => 1,
								OBJ_VAL1 => html('Track:'),
								OBJ_VAL2 => $Track{GetKey($obj, 'asm_synatk_log')} },
							{	CP2 => 1,
								OBJ_VAL1 => html('Track level:'),
								OBJ_VAL2 => $SYNTrack{GetKey($obj, 'asm_synatk_log_level')} }
						] },
						{	SECTION_END => 1 }
					);
				}
				
			} else {
				# SYNDefender < NG FP3
				push @section1, (
					{	SECTION_BEGIN => html('SYN attacks defense method:') },
					{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'fwsynatk_method') eq '0')?$MarkDot:$Mark,
									OBJ_VAL2 => html('None') },
								 {	OBJ_VAL1 => (GetKey($obj, 'fwsynatk_method') eq '1')?$MarkDot:$Mark,
									OBJ_VAL2 => html('SYN relay') },
								 {	OBJ_VAL1 => (GetKey($obj, 'fwsynatk_method') eq '3')?$MarkDot:$Mark,
									OBJ_VAL2 => html('Passive SYN gateway') }] }
				);
								
				if (GetKey($obj, 'fwsynatk_method')) {
					push @section1, (
						{	SECTION	=>	[{	CP2 =>1,
										OBJ_VAL1 => html('Timeout for SYN attack identification:'),
										OBJ_VAL2 => number(GetKey($obj, 'fwsynatk_timeout')),
										OBJ_VAL3 => html('seconds') },
									 {	CP2 => 1, INACTIVE => (GetKey($obj, 'fwsynatk_method') lt '3'),
										OBJ_VAL1 => html('Maximum protected sessions:'),
										OBJ_VAL2 => number(GetKey($obj, 'fwsynatk_max')) }] },
						{ 	SECTION	=>	[{	OBJ_VAL1 => GetKey($obj, 'fwsynatk_warning')?$CheckTick:$Check,
										OBJ_VAL2 => html('Display warning messages') }] }
					);
				}
				push @section1, { SECTION_END => 1 };
			}

			push @section1, { SF_END => 1 };
		}

	} # End of ... if ($adv_synd)
	
	# Create complete section Advanced ...
	if (@section1) {
		push @section, (
			{ SF_BEG => [
				{	SECTION  => html('Advanced'), 
					OBJ_NAME => $name, 
					STATE    => ($state eq $_OPEN) }
			] },
			@section1,
			{ SF_END => 1 }
		);
	}
	
	return @section;
	
} # end of sub Advanced

##########################################################################
# Routine: WebServer
#
# Description:
# Routine to create the WebsServer section. This section is used in 
# several object types. 
#
# Parameters:
# $rb_name    Rulebase name
# $obj        Reference to object in CheckPoint database
# $name       Name of the object
# $type       Type of the object as determined in Constants.pm
# $state      State of the section (open or closed)
#
# Returns:
# @section    An array with the section to be imported directly
#             in the templates.pm library
##########################################################################
sub WebServer {
	my $rb_name = shift;
	my $obj     = shift;
	my $name    = shift;
	my $type    = shift;
	my $state   = shift;
	
	my (@section);
	my $webobj;
	
	# Define characteristics ...
	my $gateway = (GetKey($obj, 'type') =~ /gateway/);

	# Determine version of database ...
	my $version = GetKey('FW1Objects.version');
	if ($version le '5.0') { 
		$version = 'NG';
	} elsif ($version gt '5.0') {
		$version = 'NGX';
	}
	
	if (GetKey($obj, 'additional_products:is_web_server') eq 'true') {
		
		if ($version eq "NGX") {
			
    		# Find Web server properties...
    		$webobj = GetKey($obj, 'additional_products:web_server_prop');
    		
    		push @section, (
    			{  SF_BEG => [
    				{	SECTION  => html('Web Server'), 
    					OBJ_NAME => $name, 
    					STATE    => ($state eq $_OPEN) }
    			] }
    		);
    
       		# Determine Application Engines ...
       		my (@app, $engine);
       		my @engines = GetMembers(GetKey($webobj, 'web_server_application_engines_list'));
       		foreach $engine (@engines) {
       			push @app, (
       				((GetKey($engine, 'application_engine_enforce') eq "true")?$CheckTick:$Check)." ".
       				html(stripquotes(GetKey($engine, 'application_engine_name')))
       			);
       		}
       		# Break the list in two parts ...
       		my $app1 = join('<BR>', splice(@app, 0, (int(scalar(@engines)/2 + .5))));
       		my $app2 = join('<BR>', @app);
       		push @section, (
       			{ SECTION => [
       				{	CP2 => 1,
       					OBJ_VAL1 => html("Operating System:"),
       					OBJ_VAL2 =>	$operating_system{GetKey($webobj, 'operating_system')} },
       				{	OBJ_VAL1 => html("Application Engines:"),
       					OBJ_VAL2 => $app1,
       					OBJ_VAL3 => $app2 }
       			] }
       		);
        		
    		# Port configuration ...
    		my @ports = GetMembers(GetKey($webobj, 'web_server_ports:0:additional_ports'));
    		my $ports = join('<BR>', @ports);
    		push @section, (
    			{ SECTION_BEGIN => html("Ports configurations") },
    			{ SECTION => [
    				{	CP3 => 1,
    					OBJ_VAL1 => (GetKey($webobj, 'web_server_ports:0:has_standard_port') eq "true")?$CheckTick:$Check,
    					OBJ_VAL2 => html("Server uses Standard Port:"),
    					OBJ_VAL3 => number(GetKey($webobj, 'web_server_ports:0:standard_port')) },
    				{	CP3 => 1,
    					OBJ_VAL1 => '',
    					OBJ_VAL2 => html("Server uses additional ports:"),
    					OBJ_VAL3 => $ports },
    			] },
    			{ SECTION_END => 1 }
    		);
    
    		# Protected by which firewall? ...
    		my %var = %{obj2html($rb_name, GetKey($webobj, 'protected_by_obj:Table'), GetKey($webobj, 'protected_by_obj:Name'), $_HREF)};
    		my $var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    		if ($gateway) {
        		push @section, (
        			{ SECTION_BEGIN => html("Protection") },
        			{ SECTION => [
        				{	OBJ_VAL1 => (GetKey($webobj, 'protected_by_this_obj') eq "false")?$MarkDot:$Mark,
        					OBJ_VAL2 => html("Protected by:"),
        					OBJ_VAL3 => (GetKey($webobj, 'protected_by_this_obj') eq "true")?"":$var },
        				{	OBJ_VAL1 => (GetKey($webobj, 'protected_by_this_obj') eq "true")?$MarkDot:$Mark,
        					OBJ_VAL2 => html("Protected by this machine") },
        			] },
        			{ SECTION_END => 1 }
    			);
    		} else {
        		push @section, (
        			{ SECTION_BEGIN => html("Protection") },
        			{ SECTION => [
        				{	OBJ_VAL1 => $Spacer,
        					OBJ_VAL2 => html("Protected by:"),
        					OBJ_VAL3 => $var },
        			] },
        			{ SECTION_END => 1 }
    			);
    		}			
    
    		# -------------------------------------
    		# Webserver protection ...
    		# -------------------------------------
       		push @section, (
       			{ SF_BEG => [
       				{	SECTION  => html('Protections'), 
       					SUB      => 1,
       					OBJ_NAME => $name, 
       					STATE    => 0 }
       			] },
       			{ SECTION_BEGIN => html("Web Intelligence") },
       			{ SECTION => [
       				{	CP3 => 1,
       					OBJ_VAL1 => (GetKey($webobj, 'http_buffer_overflow') eq "true")?$CheckTick:$Check,
       					OBJ_VAL2 => html("HTTP Malicious Code Protector"),
       					OBJ_VAL3 => (GetKey($webobj, 'http_buffer_overflow') eq "true")?
       								$level{GetKey($webobj, 'http_buffer_overflow_level')}:"" },
       				{	CP3 => 1,
       					OBJ_VAL1 => (GetKey($webobj, 'cross_sites_scripting') eq "true")?$CheckTick:$Check,
       					OBJ_VAL2 => html("Cross Site Scripting"),
       					OBJ_VAL3 => (GetKey($webobj, 'cross_sites_scripting') eq "true")?
       								$level{GetKey($webobj, 'strip')}:"" },
       				{	CP3 => 1,
       					OBJ_VAL1 => (GetKey($webobj, 'http_ldap_injection') eq "true")?$CheckTick:$Check,
       					OBJ_VAL2 => html("LDAP Injection"),
       					OBJ_VAL3 => (GetKey($webobj, 'http_ldap_injection') eq "true")?
       								$level{GetKey($webobj, 'http_ldap_injection_level')}:"" },			
       				{	CP3 => 1,
       					OBJ_VAL1 => (GetKey($webobj, 'http_sql_injection') eq "true")?$CheckTick:$Check,
       					OBJ_VAL2 => html("SQL Injection"),
       					OBJ_VAL3 => (GetKey($webobj, 'http_sql_injection') eq "true")?
       								$level{GetKey($webobj, 'http_sql_injection_level')}:"" },
       				{	CP3 => 1,
       					OBJ_VAL1 => (GetKey($webobj, 'http_command_stealth') eq "true")?$CheckTick:$Check,
       					OBJ_VAL2 => html("Command Injection	"),
       					OBJ_VAL3 => (GetKey($webobj, 'http_command_stealth') eq "true")?
       								$level{GetKey($webobj, 'http_command_stealth_level')}:"" },
       				{	OBJ_VAL1 => (GetKey($webobj, 'protect_http_dir_traversal') eq "true")?$CheckTick:$Check,
       					OBJ_VAL2 => html("Directory Traversal") },
       				{	OBJ_VAL1 => (GetKey($webobj, 'http_generic_header_spoofing') eq "true")?$CheckTick:$Check,
       					OBJ_VAL2 => html("Header Spoofing") },
       				{	OBJ_VAL1 => (GetKey($webobj, 'http_request_validity') eq "true")?$CheckTick:$Check,
       					OBJ_VAL2 => html("Non Ascii Headers") },
       				{	OBJ_VAL1 => (GetKey($webobj, 'http_allow_methods') eq "true")?$CheckTick:$Check,
       					OBJ_VAL2 => html("HTTP Methods") },
    			] },
    		);
    
    		if (GetKey($webobj, 'http_allow_methods') eq "true") {
        		# Advanced section of the WebServer Protections ...		
           		push @section, (
        			{ COL_BEG => 1 },
        			{ TXT => [
        				{	TEXT => $Spacer,
        					CLASS => 'note' }
        			] },
        			{ COL_NXT => 1 },
           			{ SF_BEG => [
           				{	SECTION  => html('HTTP Methods'), 
           					BUTTON   => 1,
           					OBJ_NAME => $name."HTTPMethods", 
           					STATE    => 0 }
           			] },
           			{ TXT => [
           				{	TEXT => html("Configure the protection data:"),
           					CLASS => "note" },
           			] },
           			{ SECTION => [
           				{	OBJ_VAL1 => (GetKey($webobj, 'http_allowed_methods_apply_default') eq "true")?$MarkDot:$Mark,
           					OBJ_VAL2 => html("Block default HTTP methods:") },
           				{	INACTIVE => (GetKey($webobj, 'http_allowed_methods_apply_default') eq "false"),
           					OBJ_VAL1 => "",
           					OBJ_VAL2 => ((GetKey($webobj, 'http_allowed_methods_apply_safe') eq "false")?$CheckTick:$Check).
           								html(" Block standard Safe HTTP methods") },
           				{	INACTIVE => (GetKey($webobj, 'http_allowed_methods_apply_default') eq "false"),
           					OBJ_VAL1 => "",
           					OBJ_VAL2 => ((GetKey($webobj, 'http_allowed_methods_apply_unsafe') eq "false")?$CheckTick:$Check).
           								html(" Block standard Unsafe HTTP methods") },
           				{	INACTIVE => (GetKey($webobj, 'http_allowed_methods_apply_default') eq "false"),
           					OBJ_VAL1 => "",
           					OBJ_VAL2 => ((GetKey($webobj, 'http_allowed_methods_apply_webdav') eq "false")?$CheckTick:$Check).
           								html(" Block WebDAV HTTP methods") },
           				{	OBJ_VAL1 => (GetKey($webobj, 'http_allowed_methods_apply_default') eq "false")?$MarkDot:$Mark,
           					OBJ_VAL2 => html("Block selected HTTP methods (configured globally from Web Intelligence).") },
           			] },
           			{ SF_END => 1 },
           			{ COL_END => 1 },
        		);
        	}
    
    		# Close off Protection section ...
           	push @section, (
           		{ TXT => [
           			{	TEXT => html("Note: Additional protections are configured globally from Web;Intelligence"),
           				CLASS => 'note' }
           		] },
        		{ SECTION_END => 1 },
        		{ SF_END => 1 }
        	);
    
    		# Close off WebServer section ...
       		push @section, { SF_END => 1 };
       		
       	# End of NGX section ...
       		
       	} elsif ($version eq "NG") {	# Web Server configuration in NG ...

    		# Find Web server properties...
    		$webobj = GetKey($obj, 'additional_products:web_server_prop');
    		
    		push @section, (
    			{  SF_BEG => [
    				{	SECTION  => html('Web Server'), 
    					OBJ_NAME => $name, 
    					STATE    => ($state eq $_OPEN) }
    			] }
    		);
    
			push @section, (
				{ SECTION_BEGIN => html('Ports configurations') },
				{ SECTION => [
				  	{	CP3 => 1, 
						OBJ_VAL1 => (GetKey($webobj, 'web_server_ports:0:has_standard_port') eq 'true')?$CheckTick:$Check,
						OBJ_VAL2 => html('Server uses Standard Port:'),
						OBJ_VAL3 => html('80') }
				] },
				{ SECTION_END => 1 }
			);
									
			my %var = %{obj2html($rb_name, GetKey($webobj, 'protected_by_obj:Table'), GetKey($webobj, 'protected_by_obj:Name'), $_HREF)};
			my $var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
			push @section, (
				{ SECTION_BEGIN => html('Protection') },
				{ SECTION => [
					{ 	CP2 => 1,
						OBJ_VAL1 => html('Protected by:'),
						OBJ_VAL2 => $var }
				] },
				{ SECTION_END => 1 }
			);
		
			push @section, (
				{ SECTION_BEGIN => html('Attacks defense') },
				{ SECTION => [
					{	OBJ_VAL1 => (GetKey($webobj, 'cross_sites_scripting') eq 'true')?$CheckTick:$Check,
						OBJ_VAL2 => html('Cross Site Scripting:') }
				] }
			);
			if (GetKey($webobj, 'cross_sites_scripting') eq 'true') {
				$var = GetKey($obj, 'additional_products:web_server_prop:strip');
				push @section, (
					{ SECTION => [
						{	OBJ_VAL1 => $Spacer,
							OBJ_VAL2 => ($var eq 'strip_tags')?$MarkDot:$Mark,
							OBJ_VAL3 => html('Block SCRIPT Tags') },
						{	OBJ_VAL1 => $Spacer,
							OBJ_VAL2 => ($var eq 'strip_HTML')?$MarkDot:$Mark,
							OBJ_VAL3 => html('Block all HTML Tags') },
						{	OBJ_VAL1 => $Spacer,
							OBJ_VAL2 => ($var eq 'strip_all_tags')?$MarkDot:$Mark,
							OBJ_VAL3 => html('Block all Tags') }
					] }
				);
										
			}
			push @section, { SECTION_END => 1 };
	
			push @section, { SF_END => 1 };

		} # End of Web Server for NG ...

	}
	
	return @section;
	
} # End of sub WebServer

##########################################################################
# Routine: MailServer
#
# Description:
# Routine to create the MailServer section. This section is used in 
# several object types. 
#
# Parameters:
# $rb_name    Rulebase name
# $obj        Reference to object in CheckPoint database
# $name       Name of the object
# $type       Type of the object as determined in Constants.pm
# $state      State of the section (open or closed)
#
# Returns:
# @section    An array with the section to be imported directly
#             in the templates.pm library
##########################################################################
sub MailServer {
	my $rb_name = shift;
	my $obj     = shift;
	my $name    = shift;
	my $type    = shift;
	my $state   = shift;
	
	my (@section);
	my $mailobj;
	
	if (GetKey($obj, 'additional_products:is_mail_server') eq 'true') {
		# Find Mail server properties...
		$mailobj = GetKey($obj, 'additional_products:mail_server_prop');

		push @section, (
			{  SF_BEG => [
				{	SECTION  => html('Mail Server'), 
					OBJ_NAME => $name, 
					STATE    => ($state eq $_OPEN) }
			] }
		);
		if ($operating_system{GetKey($obj, 'additional_products:operating_system')}) {
			push @section, (
				{ SECTION => [
					{	CP2 => 1,
						OBJ_VAL1 => html("Operating System:"),
						OBJ_VAL2 =>	$operating_system{GetKey($obj, 'additional_products:operating_system')} },
				] }
			);
		}
		push @section, (
			{ SECTION_BEGIN => html("User verification") },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($mailobj, 'verify_with_vpn') eq "true")?$CheckTick:$Check,
					OBJ_VAL2 => html("Verify username with VPN tunnel user") },
				{	OBJ_VAL1 => (GetKey($mailobj, 'mail_server_drop_connections') eq "true")?$CheckTick:$Check,
					OBJ_VAL2 => html("Drop connections upon verification failure") },
			] },
			{ SECTION => [
				{	CP2 => 1,
					OBJ_VAL1 => html("Track failed connections:"),
					OBJ_VAL2 => $Track{(GetKey($mailobj, 'mail_server_tracking')||"none")} },
			] },
			{ SECTION_END => 1 }
		);
		push @section, (
			{ SECTION_BEGIN => html("Servers/Ports configuration") },
			{ SECTION => [
				{	INACTIVE => !(GetKey($mailobj, 'is_pop3_server')), CP4 => 1,
					OBJ_VAL1 => (GetKey($mailobj, 'is_pop3_server') eq "true")?$CheckTick:$Check,
					OBJ_VAL2 => html("POP3 Server"),
					OBJ_VAL3 => html("Port:"),
					OBJ_VAL4 => GetKey($mailobj,'pop3_port') },
				{	INACTIVE => (GetKey($mailobj, 'is_pop3_server')), CP2 => 1,
					OBJ_VAL1 => html("POP3 Server port:"),
					OBJ_VAL2 => GetKey($mailobj,'pop3_port') },
				{	INACTIVE => !(GetKey($mailobj, 'is_imap_server')), CP4 => 1,
					OBJ_VAL1 => (GetKey($mailobj, 'is_imap_server') eq "true")?$CheckTick:$Check,
					OBJ_VAL2 => html("IMAP Server"),
					OBJ_VAL3 => html("Port:"),
					OBJ_VAL4 => GetKey($mailobj,'imap_port') },
				{	INACTIVE => (GetKey($mailobj, 'is_imap_server')), CP2 => 1,
					OBJ_VAL1 => html("IMAP Server port:"),
					OBJ_VAL2 => GetKey($mailobj,'imap_port') },
				{	INACTIVE => !(GetKey($mailobj, 'is_smtp_server')), CP4 => 1,
					OBJ_VAL1 => (GetKey($mailobj, 'is_smtp_server') eq "true")?$CheckTick:$Check,
					OBJ_VAL2 => html("SMTP Server"),
					OBJ_VAL3 => html("Port:"),
					OBJ_VAL4 => GetKey($mailobj,'smtp_port') },
			] },
			{ SECTION_END => 1 }
		);
		push @section, (
			{ SECTION_BEGIN => html("Logging") },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($mailobj, 'log_username') eq "true")?$CheckTick:$Check,
					OBJ_VAL2 => html("Log POP3/IMAP username") },
			] },
			{ SECTION_END => 1 }
		);
		# -------------------------------------
		# Mailserver protection ...
		# -------------------------------------
   		push @section, (
   			{ SF_BEG => [
   				{	SECTION  => html('Protections'), 
   					SUB      => 1,
   					OBJ_NAME => $name.'MailServer', 
   					STATE    => 0 }
   			] },
   			{ SECTION_BEGIN => html("SmartDefense") },
   			{ TXT => [
   				{	TEXT => html("Note: The enforcement of the settings on this page is;").
   									html("dependant on the profile that protects this server."),
   					CLASS => 'note' }
   			] },
   			{ SECTION => [
   				{	OBJ_VAL1 => (GetKey($mailobj, 'pop3_imap_security') eq "true")?$CheckTick:$Check,
   					OBJ_VAL2 => html("POP3/IMAP Security") },
   			] },
   			{ SECTION_END => 1 },
   			{ SF_END => 1 }
   		);
   		

		# Close of Mail Server section ...
		push @section, { SF_END=> 1 };
	}
	
	return @section;
	
} # End of sub MailServer


##########################################################################
# Routine: DNSServer
#
# Description:
# Routine to create the DNSServer section. This section is used in 
# several object types. 
#
# Parameters:
# $rb_name    Rulebase name
# $obj        Reference to object in CheckPoint database
# $name       Name of the object
# $type       Type of the object as determined in Constants.pm
# $state      State of the section (open or closed)
#
# Returns:
# @section    An array with the section to be imported directly
#             in the templates.pm library
##########################################################################
sub DNSServer {
	my $rb_name = shift;
	my $obj     = shift;
	my $name    = shift;
	my $type    = shift;
	my $state   = shift;
	
	my (@section);
	my $dnsobj;
	my (%var,$var);
	
	# Define characteristics ...
	my $gateway = (GetKey($obj, 'type') =~ /gateway/);

	if (GetKey($obj, 'additional_products:is_dns_server') eq 'true') {
		# Find DNS server properties...
		$dnsobj = GetKey($obj, 'additional_products:dns_server_prop');

		push @section, (
			{  SF_BEG => [
				{	SECTION  => html('DNS Server'), 
					OBJ_NAME => $name, 
					STATE    => ($state eq $_OPEN) }
			] }
		);
		
    	#------------------------------------------------
    	# Determine DNS objects ...
    	#------------------------------------------------
    	my @domains = GetMembers(GetKey($dnsobj, 'authorization_domains_list'));
       	my @section1;

    	if (@domains) {
    		# Create HTML list of objects ...
    		foreach $var (@domains) {
    			%var = %{obj2html($rb_name, GetKey($var, 'Table'), GetKey($var, 'Name'), $_HREF)};
    			$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    			push @section1, { OBJ_VAL1 => $var };
    		}
    	} else {
    		push @section1, { OBJ_VAL1 => $None }
    	}
    			
    	push @section, (
    		{ SECTION_BEGIN => html('Authorization domains list:') },
    		{ SECTION => \@section1 },
    		{ SECTION_END => 1 }
    	);
    	undef @domains;

 		# Protected by which firewall? ...
		my %var = %{obj2html($rb_name, GetKey($dnsobj, 'protected_by_obj:Table'), GetKey($dnsobj, 'protected_by_obj:Name'), $_HREF)};
		my $var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
		if ($gateway) {
    		push @section, (
    			{ SECTION_BEGIN => html("Protection") },
    			{ SECTION => [
    				{	OBJ_VAL1 => (GetKey($dnsobj, 'protected_by_this_obj') eq "false")?$MarkDot:$Mark,
    					OBJ_VAL2 => html("Protected by:"),
    					OBJ_VAL3 => (GetKey($dnsobj, 'protected_by_this_obj') eq "true")?"":$var },
    				{	OBJ_VAL1 => (GetKey($dnsobj, 'protected_by_this_obj') eq "true")?$MarkDot:$Mark,
    					OBJ_VAL2 => html("Protected by this machine") },
    			] },
    			{ SECTION_END => 1 }
			);
		} else {
    		push @section, (
    			{ SECTION_BEGIN => html("Protection") },
    			{ SECTION => [
    				{	OBJ_VAL1 => $Spacer,
    					OBJ_VAL2 => html("Protected by:"),
    					OBJ_VAL3 => $var },
    			] },
    			{ SECTION_END => 1 }
			);
		}			

		# -------------------------------------
		# DNSserver protection ...
		# -------------------------------------
   		push @section, (
   			{ SF_BEG => [
   				{	SECTION  => html('Protections'), 
   					SUB      => 1,
   					OBJ_NAME => $name.'DNSServer', 
   					STATE    => 0 }
   			] },
   			{ SECTION_BEGIN => html("SmartDefense") },
   			{ SECTION => [
   				{	OBJ_VAL1 => (GetKey($dnsobj, 'randomization') eq "true")?$CheckTick:$Check,
   					OBJ_VAL2 => html("Scramble source port and session id") },
   				{	OBJ_VAL1 => (GetKey($dnsobj, 'external_domain_requests') eq "true")?$CheckTick:$Check,
   					OBJ_VAL2 => html("Drop inbound requests for external domains") },
   			] },
   			{ SECTION_END => 1 },
   			{ TXT => [
   				{	TEXT => html("Note: Additional protections are configured globally;from SmartDefense."),
   					CLASS => 'note' }
   			] },
   			{ SF_END => 1 }
   		);
   		
		# Close of DNS Server section ...
		push @section, { SF_END=> 1 };
	}
	
	return @section;
	
} # End of sub DNSServer
    

1;